@@ -5,25 +5,17 @@ from typing import Optional
 
 import click
 from click_option_group import MutuallyExclusiveOptionGroup, optgroup
+from evernote.edam.error.ttypes import EDAMErrorCode, EDAMSystemException
 
-from evernote_backup import cli_app
+from evernote_backup import cli_app, config_defaults
 from evernote_backup.cli_app_click_util import (
     DIR_ONLY,
     FILE_ONLY,
     NaturalOrderGroup,
     group_options,
 )
-from evernote_backup.cli_app_util import ProgramTerminatedError, is_output_to_terminal
-from evernote_backup.config_defaults import (
-    BACKEND,
-    DATABASE_NAME,
-    NETWORK_ERROR_RETRY_COUNT,
-    OAUTH_LOCAL_PORT,
-    SYNC_CHUNK_MAX_RESULTS,
-    SYNC_CHUNK_MAX_RESULTS_SERVER_LIMIT,
-    SYNC_MAX_DOWNLOAD_WORKERS,
-    SYNC_MAX_DOWNLOAD_WORKERS_SANE_LIMIT,
-)
+from evernote_backup.cli_app_util import ProgramTerminatedError
+from evernote_backup.log_util import get_time_txt, init_logging, init_logging_format
 from evernote_backup.version import __version__
 
 logger = logging.getLogger()
@@ -54,7 +46,7 @@ opt_oauth = click.option(
 
 opt_oauth_port = click.option(
     "--oauth-port",
-    default=OAUTH_LOCAL_PORT,
+    default=config_defaults.OAUTH_LOCAL_PORT,
     show_default=True,
     help="OAuth local server port. (Advanced option, use with --oauth.)",
 )
@@ -70,7 +62,7 @@ opt_token = click.option(
 
 opt_network_retry_count = click.option(
     "--network-retry-count",
-    default=NETWORK_ERROR_RETRY_COUNT,
+    default=config_defaults.NETWORK_ERROR_RETRY_COUNT,
     show_default=True,
     type=click.IntRange(1),
     help=("Network error retry count. (Advanced option)"),
@@ -79,7 +71,7 @@ opt_network_retry_count = click.option(
 opt_database = click.option(
     "--database",
     "-d",
-    default=DATABASE_NAME,
+    default=config_defaults.DATABASE_NAME,
     show_default=True,
     required=True,
     type=FILE_ONLY,
@@ -112,16 +104,8 @@ def cli(quiet: bool, verbose: bool) -> None:
     evernote-backup export output_dir/
     """
 
-    if is_output_to_terminal():
-        log_format = "%(message)s"
-    else:
-        log_format = "%(asctime)s | [%(levelname)s] | %(message)s"
-
-    for h in logging.root.handlers[:]:
-        logging.root.removeHandler(h)
-        h.close()
-
-    logging.basicConfig(format=log_format)
+    init_logging()
+    init_logging_format()
 
     if quiet:
         logger.setLevel(logging.CRITICAL)
@@ -141,7 +125,7 @@ def cli(quiet: bool, verbose: bool) -> None:
 )
 @click.option(
     "--backend",
-    default=BACKEND,
+    default=config_defaults.BACKEND,
     show_default=True,
     type=click.Choice(["evernote", "evernote:sandbox", "china", "china:sandbox"]),
     help="API backend to connect to. If you are using Yinxiang, select 'china'.",
@@ -177,19 +161,29 @@ def init_db(
 @opt_database
 @click.option(
     "--max-chunk-results",
-    default=SYNC_CHUNK_MAX_RESULTS,
-    type=click.IntRange(1, SYNC_CHUNK_MAX_RESULTS_SERVER_LIMIT),
+    default=config_defaults.SYNC_CHUNK_MAX_RESULTS,
+    type=click.IntRange(1, config_defaults.SYNC_CHUNK_MAX_RESULTS_SERVER_LIMIT),
     show_default=True,
     help="Max entries per sync chunk. (Advanced option)",
 )
 @click.option(
     "--max-download-workers",
-    default=SYNC_MAX_DOWNLOAD_WORKERS,
+    default=config_defaults.SYNC_MAX_DOWNLOAD_WORKERS,
     show_default=True,
-    type=click.IntRange(1, SYNC_MAX_DOWNLOAD_WORKERS_SANE_LIMIT),
+    type=click.IntRange(1, config_defaults.SYNC_MAX_DOWNLOAD_WORKERS_SANE_LIMIT),
     help=(
-        "Max number of parallel downloads."
-        " (Advanced option, don't set too high to avoid rate limits.)"
+        "Max number of parallel downloads. Don't set too high to avoid rate limits."
+        " (Advanced option)"
+    ),
+)
+@click.option(
+    "--download-cache-memory-limit",
+    default=config_defaults.SYNC_DOWNLOAD_CACHE_MEMORY_LIMIT,
+    show_default=True,
+    type=click.IntRange(1),
+    help=(
+        "Cache size in MB for notes stored in memory before writing to disk."
+        " (Advanced option)"
     ),
 )
 @opt_network_retry_count
@@ -197,6 +191,7 @@ def sync(
     database: str,
     max_chunk_results: int,
     max_download_workers: int,
+    download_cache_memory_limit: int,
     network_retry_count: int,
 ) -> None:
     """Sync local database with Evernote, downloading all notes."""
@@ -205,6 +200,7 @@ def sync(
         database=database,
         max_chunk_results=max_chunk_results,
         max_download_workers=max_download_workers,
+        download_cache_memory_limit=download_cache_memory_limit,
         network_retry_count=network_retry_count,
     )
 
@@ -277,6 +273,15 @@ def main() -> None:
     except ProgramTerminatedError as e:
         logger.critical(e)
         sys.exit(1)
+    except EDAMSystemException as e:
+        if e.errorCode != EDAMErrorCode.RATE_LIMIT_REACHED:
+            logger.critical(traceback.format_exc())
+            sys.exit(1)
+
+        time_left = get_time_txt(e.rateLimitDuration)
+
+        logger.critical(f"Rate limit reached. Restart program in {time_left}.")
+        sys.exit(1)
     except Exception:
         logger.critical(traceback.format_exc())
         sys.exit(1)
